'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var ListboxContext = /*#__PURE__*/React__default.createContext({});
var Listbox = /*#__PURE__*/function (_Component) {
  _inheritsLoose(Listbox, _Component);

  function Listbox(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _this.getActiveDescendant = function () {
      var _ref = _this.state.optionIds.find(function (_ref2) {
        var value = _ref2[0];
        return value === _this.state.activeItem;
      }) || [null, null],
          id = _ref[1];

      return id;
    };

    _this.registerOptionId = function (value, optionId) {
      _this.unregisterOptionId(value);

      _this.setState(function (prevState) {
        return {
          optionIds: [].concat(prevState.optionIds, [[value, optionId]])
        };
      });
    };

    _this.unregisterOptionId = function (value) {
      _this.setState(function (prevState) {
        return {
          optionIds: prevState.optionIds.filter(function (_ref3) {
            var candidateValue = _ref3[0];
            return candidateValue !== value;
          })
        };
      });
    };

    _this.type = function (value) {
      _this.setState(function (prevState) {
        return {
          typeahead: prevState.typeahead.concat(value)
        };
      }, function () {
        var _ref4 = _this.state.optionRefs.find(function (_ref5) {
          var ref = _ref5[1];
          var el = ref;
          return el.innerText.toLowerCase().startsWith(_this.state.typeahead.toLowerCase());
        }) || [null],
            match = _ref4[0];

        if (match !== null) {
          _this.focus(match);
        }

        _this.clearTypeahead();
      });
    };

    _this.clearTypeahead = function () {
      setTimeout(function () {
        _this.setState({
          typeahead: ""
        });
      }, 500);
    };

    _this.registerOptionRef = function (value, optionRef) {
      _this.unregisterOptionRef(value);

      _this.setState(function (prevState) {
        return {
          optionRefs: [].concat(prevState.optionRefs, [[value, optionRef]])
        };
      });
    };

    _this.unregisterOptionRef = function (value) {
      _this.setState(function (prevState) {
        return {
          optionRefs: prevState.optionRefs.filter(function (_ref6) {
            var candidateValue = _ref6[0];
            return candidateValue !== value;
          })
        };
      });
    };

    _this.toggle = function () {
      _this.state.isOpen ? _this.close() : _this.open();
    };

    _this.open = function () {
      _this.setState({
        isOpen: true
      }, function () {
        process.nextTick(function () {
          if (_this.state.listboxListRef) {
            var activeValue = _this.props.value; // Set active value to be the first option
            // in the list if no item is selected.
            // https://www.w3.org/TR/wai-aria-practices/#listbox_kbd_interaction

            if (!activeValue) {
              activeValue = _this.state.values[0];
            }

            _this.focus(activeValue);

            process.nextTick(function () {
              var _this$state$listboxLi;

              (_this$state$listboxLi = _this.state.listboxListRef) === null || _this$state$listboxLi === void 0 ? void 0 : _this$state$listboxLi.focus();
            });
          }
        });
      });
    };

    _this.close = function () {
      _this.setState({
        isOpen: false
      }, function () {
        var _this$state$listboxBu;

        (_this$state$listboxBu = _this.state.listboxButtonRef) === null || _this$state$listboxBu === void 0 ? void 0 : _this$state$listboxBu.focus();
      });
    };

    _this.select = function (value) {
      _this.props.onChange(value);

      process.nextTick(function () {
        _this.close();
      });
    };

    _this.focus = function (value) {
      _this.setState({
        activeItem: value
      }, function () {
        var _this$state$listboxLi2;

        if (value === null) {
          return;
        }

        (_this$state$listboxLi2 = _this.state.listboxListRef) === null || _this$state$listboxLi2 === void 0 ? void 0 : _this$state$listboxLi2.children[_this.state.values.indexOf(_this.state.activeItem)].scrollIntoView({
          block: "nearest"
        });
      });
    };

    _this.setListboxButtonRef = function (ref) {
      _this.setState({
        listboxButtonRef: ref
      });
    };

    _this.setListboxListRef = function (ref) {
      _this.setState({
        listboxListRef: ref
      });
    };

    _this.setButtonId = function (id) {
      _this.setState({
        buttonId: id
      });
    };

    _this.setLabelId = function (id) {
      _this.setState({
        labelId: id
      });
    };

    _this.setValues = function (values) {
      _this.setState({
        values: values
      });
    };

    _this.setActiveItem = function (activeItem) {
      _this.setState({
        activeItem: activeItem
      });
    };

    _this.state = {
      typeahead: "",
      listboxButtonRef: null,
      listboxListRef: null,
      isOpen: false,
      activeItem: _this.props.value,
      values: [],
      labelId: null,
      buttonId: null,
      optionIds: [],
      optionRefs: []
    };
    return _this;
  }

  var _proto = Listbox.prototype;

  _proto.render = function render() {
    var _this$props = this.props,
        children = _this$props.children,
        className = _this$props.className;
    var isOpen = this.state.isOpen;
    var ProvidedContext = {
      getActiveDescendant: this.getActiveDescendant,
      registerOptionId: this.registerOptionId,
      unregisterOptionId: this.unregisterOptionId,
      registerOptionRef: this.registerOptionRef,
      unregisterOptionRef: this.unregisterOptionRef,
      toggle: this.toggle,
      open: this.open,
      close: this.close,
      select: this.select,
      focus: this.focus,
      clearTypeahead: this.clearTypeahead,
      typeahead: this.state.typeahead,
      type: this.type,
      setListboxListRef: this.setListboxListRef,
      setListboxButtonRef: this.setListboxButtonRef,
      listboxButtonRef: this.state.listboxButtonRef,
      listboxListRef: this.state.listboxListRef,
      isOpen: this.state.isOpen,
      activeItem: this.state.activeItem,
      setActiveItem: this.setActiveItem,
      values: this.state.values,
      setValues: this.setValues,
      labelId: this.state.labelId,
      setLabelId: this.setLabelId,
      buttonId: this.state.buttonId,
      setButtonId: this.setButtonId,
      props: this.props
    };
    return React__default.createElement(ListboxContext.Provider, {
      value: ProvidedContext
    }, React__default.createElement("div", {
      className: className
    }, children instanceof Function ? children({
      isOpen: isOpen
    }) : children));
  };

  return Listbox;
}(React.Component);

var id = 0;
var generateId = function generateId() {
  return "tailwind-ui-listbox-id-" + ++id;
};

var ListboxButton = /*#__PURE__*/function (_Component) {
  _inheritsLoose(ListboxButton, _Component);

  function ListboxButton(props) {
    var _this;

    _this = _Component.call(this, props) || this;
    _this.ownRef = null;

    _this.focus = function () {
      return _this.setState({
        isFocused: true
      });
    };

    _this.blur = function () {
      return _this.setState({
        isFocused: false
      });
    };

    _this.state = {
      id: generateId(),
      isFocused: false
    };
    return _this;
  }

  var _proto = ListboxButton.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.context.setButtonId(this.state.id);
    this.context.setListboxButtonRef(this.ownRef);
  };

  _proto.render = function render() {
    var _this2 = this;

    var _this$props = this.props,
        children = _this$props.children,
        className = _this$props.className;
    var isFocused = this.state.isFocused;
    return React__default.createElement("button", Object.assign({
      "aria-haspopup": "listbox",
      "aria-labelledby": this.context.labelId + " " + this.state.id,
      className: className,
      id: this.state.id,
      onBlur: this.blur,
      onClick: this.context.toggle,
      onFocus: this.focus,
      ref: function ref(el) {
        return _this2.ownRef = el;
      },
      type: "button"
    }, this.context.isOpen ? {
      "aria-expanded": "true"
    } : {}), children instanceof Function ? children({
      isFocused: isFocused
    }) : children);
  };

  return ListboxButton;
}(React.Component);
ListboxButton.contextType = ListboxContext;

var ListboxLabel = /*#__PURE__*/function (_Component) {
  _inheritsLoose(ListboxLabel, _Component);

  function ListboxLabel(props) {
    var _this;

    _this = _Component.call(this, props) || this;
    _this.state = {
      id: generateId()
    };
    return _this;
  }

  var _proto = ListboxLabel.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.context.setLabelId(this.state.id);
  };

  _proto.render = function render() {
    var _this$props = this.props,
        children = _this$props.children,
        className = _this$props.className;
    return children && React__default.createElement("span", {
      className: className,
      id: this.state.id
    }, children);
  };

  return ListboxLabel;
}(React.Component);
ListboxLabel.contextType = ListboxContext;

function isString(value) {
  return typeof value === "string" || value instanceof String;
}

var ListboxList = /*#__PURE__*/function (_Component) {
  _inheritsLoose(ListboxList, _Component);

  function ListboxList(props) {
    var _this;

    _this = _Component.call(this, props) || this;
    _this.ownRef = null;

    _this.handleBlur = function (e) {
      if (e.relatedTarget === _this.context.listboxButtonRef) {
        return;
      } // The button will already handle the toggle for us


      _this.context.close();
    };

    _this.handleKeydown = function (e) {
      var _this$state$values;

      var focusedIndex = (_this$state$values = _this.state.values) === null || _this$state$values === void 0 ? void 0 : _this$state$values.indexOf(_this.context.activeItem); // focusedIndex is -1 if this.context.activeItem  is not in this.state.values

      if (focusedIndex === -1 || !_this.state.values) {
        return;
      }

      var indexToFocus;

      switch (e.key) {
        case "Esc":
        case "Escape":
          e.preventDefault();

          _this.context.close();

          break;

        case "Tab":
          e.preventDefault();

          _this.context.close();

          break;

        case "End":
          e.preventDefault();

          _this.context.focus(_this.state.values[_this.state.values.length - 1]);

          break;

        case "Home":
          e.preventDefault();

          _this.context.focus(_this.state.values[0]);

          break;

        case "Up":
        case "ArrowUp":
          e.preventDefault();

          if (focusedIndex || focusedIndex === 0) {
            var _this$state$values2;

            // Typescript makes us check this.
            indexToFocus = focusedIndex - 1 < 0 ? ((_this$state$values2 = _this.state.values) === null || _this$state$values2 === void 0 ? void 0 : _this$state$values2.length) - 1 : focusedIndex - 1;

            _this.context.focus(_this.state.values[indexToFocus]);
          }

          break;

        case "Down":
        case "ArrowDown":
          e.preventDefault();

          if (focusedIndex || focusedIndex === 0) {
            var _this$state$values3;

            // Typescript makes us check this.
            indexToFocus = focusedIndex + 1 > ((_this$state$values3 = _this.state.values) === null || _this$state$values3 === void 0 ? void 0 : _this$state$values3.length) - 1 ? 0 : focusedIndex + 1;

            _this.context.focus(_this.state.values[indexToFocus]);
          }

          break;

        case "Spacebar":
        case " ":
          e.preventDefault();

          if (_this.context.typeahead !== "") {
            _this.context.type(" ");
          } else {
            _this.context.select(_this.context.activeItem || _this.context.props);
          }

          break;

        case "Enter":
          e.preventDefault();

          _this.context.select(_this.context.activeItem || _this.context.props);

          break;

        default:
          if (!(isString(e.key) && e.key.length === 1)) {
            return;
          }

          e.preventDefault();

          _this.context.type(e.key);

          return;
      }
    };

    _this.handleMouseLeave = function () {
      _this.context.setActiveItem(null);
    };

    _this.state = {
      focusedIndex: null,
      values: null
    };
    return _this;
  }

  var _proto = ListboxList.prototype;

  _proto.componentDidMount = function componentDidMount() {
    var _this2 = this;

    this.context.setListboxListRef(this.ownRef);
    var values = this.props.children.map(function (node) {
      return node.props.value;
    });
    this.setState({
      values: values
    }, function () {
      _this2.context.setValues(values);
    });
  };

  _proto.render = function render() {
    var _this3 = this;

    var _this$props = this.props,
        children = _this$props.children,
        className = _this$props.className;
    return React__default.createElement("ul", {
      "aria-activedescendant": this.context.getActiveDescendant(),
      "aria-labelledby": this.context.props.labelledby,
      className: className,
      onBlur: this.handleBlur,
      onKeyDown: this.handleKeydown,
      onMouseLeave: this.handleMouseLeave,
      ref: function ref(el) {
        return _this3.ownRef = el;
      },
      role: "listbox",
      tabIndex: -1
    }, children);
  };

  return ListboxList;
}(React.Component);
ListboxList.contextType = ListboxContext;

var ListboxOption = /*#__PURE__*/function (_Component) {
  _inheritsLoose(ListboxOption, _Component);

  function ListboxOption(props) {
    var _this;

    _this = _Component.call(this, props) || this;
    _this.ownRef = null;

    _this.handleClick = function () {
      _this.context.select(_this.props.value);
    };

    _this.handleMouseMove = function () {
      if (_this.context.activeItem === _this.props.value) {
        return;
      } // this option is already active


      _this.context.setActiveItem(_this.props.value);
    };

    _this.state = {
      id: generateId()
    };
    return _this;
  }

  var _proto = ListboxOption.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.context.registerOptionRef(this.props.value, this.ownRef);
    this.context.registerOptionId(this.props.value, this.state.id);
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.context.unregisterOptionId(this.props.value);
    this.context.unregisterOptionRef(this.props.value);
  };

  _proto.render = function render() {
    var _this2 = this;

    var _this$props = this.props,
        children = _this$props.children,
        className = _this$props.className;
    var isActive = this.context.activeItem === this.props.value;
    var isSelected = this.context.props.value === this.props.value;
    return React__default.createElement("li", Object.assign({
      className: className,
      id: this.state.id,
      onMouseDown: this.handleClick,
      onMouseMove: this.handleMouseMove,
      ref: function ref(el) {
        return _this2.ownRef = el;
      },
      role: "option",
      tabIndex: -1
    }, isSelected ? {
      "aria-selected": true
    } : {}), children instanceof Function ? children({
      isSelected: isSelected,
      isActive: isActive
    }) : children);
  };

  return ListboxOption;
}(React.Component);
ListboxOption.contextType = ListboxContext;

exports.Listbox = Listbox;
exports.ListboxButton = ListboxButton;
exports.ListboxLabel = ListboxLabel;
exports.ListboxList = ListboxList;
exports.ListboxOption = ListboxOption;
//# sourceMappingURL=react-listbox.cjs.development.js.map
